package eu.mhsl.minenet.minigames.instance.game.minigame.config;

import eu.mhsl.minenet.minigames.instance.game.Game;
import eu.mhsl.minenet.minigames.shared.inventory.InteractableInventory;
import eu.mhsl.minenet.minigames.util.TextUtil;
import eu.mhsl.minenet.minigames.instance.room.Room;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.minestom.server.entity.Player;
import net.minestom.server.inventory.InventoryType;
import net.minestom.server.inventory.click.ClickType;
import net.minestom.server.inventory.condition.InventoryConditionResult;
import net.minestom.server.item.ItemStack;
import net.minestom.server.item.Material;

import java.util.HashMap;
import java.util.Map;

public class GameConfigurationInventory extends InteractableInventory {
    private final Map<Integer, Option<?>> map = new HashMap<>();

    public GameConfigurationInventory(GameFactory factory) {
        super(InventoryType.CHEST_5_ROW, factory.name());
        ConfigManager config = factory.configuration();

        setClickableItem(
                ItemStack.builder(Material.RED_WOOL).displayName(Component.text("Abbrechen", NamedTextColor.RED)).build(),
                0,
                itemClick -> itemClick.getPlayer().closeInventory(),
                true
        );

        setDummyItem(Material.BLACK_STAINED_GLASS_PANE,1);

        setDummyItem(
                ItemStack.builder(Material.NAME_TAG).displayName(factory.name()).build(),
                4
        );

        setDummyItem(Material.BLACK_STAINED_GLASS_PANE,7);

        setClickableItem(
                ItemStack.builder(Material.GREEN_WOOL).displayName(Component.text("Start", NamedTextColor.GREEN)).build(),
                8,
                itemClick -> {
                    try {
                        Game game = factory.manufacture(config != null ? config.getAll() : null);
                        Room.getRoom(itemClick.getPlayer()).moveMembersToGame(game);
                        game.load();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                },
                true
        );

        for(int i = 9; i <= 17; i++) {
            setDummyItem(Material.BLACK_STAINED_GLASS_PANE, i);
        }

        if(config == null) {
            setDummyItem(
                    ItemStack.builder(Material.BARRIER).displayName(Component.text("Keine Optionen")).lore(TextUtil.autoWrap("Für dieses Spiel sind keine Einstellungen verfügbar!")).build(),
                    31
            );
            return;
        }

        int pos = 18;
        for(Option<?> item : config.getAll()) {
            map.put(pos, item);

            setDummyItem(
                    item.getCurrent(),
                    pos++
            );
        }
    }

    @Override
    protected void onClick(Player player, int slot, ClickType clickType, InventoryConditionResult inventoryConditionResult) {
        inventoryConditionResult.setCancel(true);

        if(!map.containsKey(slot)) return;

        Option item = map.get(slot);
        setDummyItem(
                item.getNext(),
                slot
        );

        update();

    }
}
